//
//  MSAssetTrackingMapViewController.swift
//  MeridianSwiftSamples
//
//  Created by miedema on 8/11/18.
//  Copyright © 2018 Aruba Networks. All rights reserved.
//

import Foundation

class MSAssetTrackingMapViewController : MRMapViewController {
    // MARK: - Properties
    lazy var toolBar: UIToolbar = {
        let toolbar = UIToolbar(frame: .zero)
        toolbar.translatesAutoresizingMaskIntoConstraints = false
        return toolbar
    }()

    lazy var toolBarItems: [UIBarButtonItem] = {
        return [removeSubscriptionBarButton,
                addSubscriptionBarButtonItem]
    }()

    lazy var removeSubscriptionBarButton: UIBarButtonItem = {
        return UIBarButtonItem(title: "Remove Subscription", style: .plain, target: self, action: .removeSubscriptionAction)
    }()

    lazy var addSubscriptionBarButtonItem: UIBarButtonItem = {
        return UIBarButtonItem(title: "Add Subscription", style: .plain, target: self, action: .showAvailableSubscriptionsAction)
    }()
}


// MARK: - View Lifecycle
extension MSAssetTrackingMapViewController {
    override func viewDidLoad() {
        super.viewDidLoad()
        mapView.mapKey = MREditorKey(forMap:MSExampleHost.mapID(),
                                     app:MSExampleHost.appID())

        mapView.showsTags = true
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        view.addSubview(toolBar)
        toolBar.leadingAnchor.constraint(equalTo: view.leadingAnchor).isActive = true
        toolBar.trailingAnchor.constraint(equalTo: view.trailingAnchor).isActive = true
        if #available(iOS 11.0, *) {
            toolBar.bottomAnchor.constraint(equalTo: view.safeAreaLayoutGuide.bottomAnchor).isActive = true
        } else {
            toolBar.bottomAnchor.constraint(equalTo: view.bottomAnchor).isActive = true
        }
        toolBar.heightAnchor.constraint(equalToConstant: 44).isActive = true

        toolBar.setItems(toolBarItems, animated: true)
    }
}

// MARK: - Actions
private extension Selector {
    static let removeSubscriptionAction = #selector(MSAssetTrackingMapViewController.removeSubscription(_:))
    static let showAvailableSubscriptionsAction = #selector(MSAssetTrackingMapViewController.showAvailableSubscriptions(_:))
}

extension MSAssetTrackingMapViewController {
    @objc func removeSubscription(_ sender: UIBarButtonItem) {
        let sheetView = UIAlertController(title: "Remove Subscription", message: nil, preferredStyle: .actionSheet)
        sheetView.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))

        for subscription in mapView.tagSubscriptions {
            let title: String

            if subscription is MRDTagIdentifierSubscription {
                title = "Identifier"
            } else if subscription is MRDMapTagSubscription {
                title = "Map"
            } else if subscription is MRDTagLabelSubscription {
                title = "Label"
            } else {
                fatalError("Unhandled subscription type \(subscription)")
            }

            sheetView.addAction(UIAlertAction(title: "\(title) - \(subscription.identifier)", style: .destructive, handler: { (action) in
                self.mapView.tagSubscriptions = self.mapView.tagSubscriptions.filter({
                    !$0.isEqual(subscription)
                })
            }))
        }

        present(sheetView, animated: true, completion: nil)
    }

    @objc func showAvailableSubscriptions(_ sender: UIBarButtonItem) {
        let sheetView = UIAlertController(title: "Add Subsciption", message: nil, preferredStyle: .actionSheet)
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        sheetView.addAction(cancelAction)

        let currentMapAction = UIAlertAction(title: "Current Map", style: .default, handler: { (action) in
            let subscription = MRDMapTagSubscription(identifier: self.mapView.mapKey.identifier)
            self.mapView.tagSubscriptions = [subscription]
        })
        let labelIDAction = UIAlertAction(title: "Label ID", style: .default, handler: { (action) in
            let labelIDAlert = UIAlertController(title: "Label ID", message: nil, preferredStyle: .alert)
            labelIDAlert.addAction(cancelAction)
            labelIDAlert.addTextField(configurationHandler: { (textField) in })
            labelIDAlert.addAction(UIAlertAction(title: "Add", style: .default, handler: { (action) in
                guard let id = labelIDAlert.textFields?.first?.text else {
                    return
                }
                self.mapView.tagSubscriptions += [MRDTagLabelSubscription(identifier: id)]
            }))
            self.present(labelIDAlert, animated: true, completion: nil)
        })
        let identifierAction = UIAlertAction(title: "Asset Identifier", style: .default, handler: { (action) in
            let tagIDAlert = UIAlertController(title: "Label ID", message: nil, preferredStyle: .alert)
            tagIDAlert.addAction(cancelAction)
            tagIDAlert.addTextField(configurationHandler: { (textField) in })
            tagIDAlert.addAction(UIAlertAction(title: "Add", style: .default, handler: { (action) in
                guard let id = tagIDAlert.textFields?.first?.text else {
                    return
                }
                self.mapView.tagSubscriptions += [MRDTagIdentifierSubscription(identifier: id)]
            }))
            self.present(tagIDAlert, animated: true, completion: nil)
        })

        if mapView.tagSubscriptions.isEmpty {
            sheetView.addAction(currentMapAction)
            sheetView.addAction(labelIDAction)
            sheetView.addAction(identifierAction)
        } else if mapView.tagSubscriptions.filter({
            // we only support multiple subscriptions when they are all for Label IDs
            $0 is MRDMapTagSubscription || $0 is MRDTagIdentifierSubscription
        }).isEmpty {
            sheetView.addAction(labelIDAction)
        }

        present(sheetView, animated: true, completion: nil)
    }
}


// MARK: - MSExampleViewController
extension MSAssetTrackingMapViewController : MSExampleViewController {
    static func exampleTitle() -> String {
        return "Asset tracking Map View Controller"
    }
    static func exampleDetail() -> String {
        return "Example usage of asset tracking subscriptions"
    }
}
