//
//  MSOverlayViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSOverlayViewController : UIViewController, MRMapViewDelegate, MSExampleViewController {
    private lazy var mapView : MRMapView = {
        let mapView = MRMapView()
        mapView.delegate = self
        mapView.mapKey = MREditorKey(forMap : MSExampleHost.mapID(), app : MSExampleHost.appID())
        return mapView
    } ()
    
    private var circleOverlay : MRPathOverlay?
    private var pathOverlay : MRPathOverlay?
    private var placemarkOverlays : Dictionary<MREditorKey, MRPathOverlay> = [:]
    
    private var blueOpaqueStroke = UIColor(red : 0.000, green : 0.000, blue : 1.000, alpha : 1)
    private var redOpaqueStroke = UIColor(red : 1.000, green : 0.000, blue : 0.000, alpha : 1)
    private var blueTransparentStroke = UIColor(red : 0.000, green : 0.000, blue : 1.000, alpha : 0.400)
    private var redTransparentStroke = UIColor(red : 1.000, green : 0.000, blue : 0.000, alpha : 0.400)
    private var blueTransparentFill = UIColor(red : 0.000, green : 0.000, blue : 1.000, alpha : 0.200)
    private var redTransparentFill = UIColor(red : 1.000, green : 0.000, blue : 0.000, alpha : 0.200)
    private var clearFill = UIColor(red : 0.000, green : 0.000, blue : 0.000, alpha : 0.000)
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.view = self.mapView
        
        let button0 = UIBarButtonItem(title : "Toggle Color", style :.plain, target : self, action : #selector(self.toggleColor))
        let button1 = UIBarButtonItem(title : "Toggle Overlay", style :.plain, target : self, action : #selector(self.toggleOverlay))
        self.navigationItem.rightBarButtonItems = [button0, button1]
        
        circleOverlay = nil
        pathOverlay = nil
        
        toggleOverlay()
    }
    
    // MARK: - MSExampleViewController
    
    static func exampleTitle()->String {
        return "Custom Overlays"
    }
    
    static func exampleDetail()->String {
        return "Example of how to add custom overlays.\n\n" +
        "• Button to toggle the appearance of the overlays.\n"
        "• Button to toggle the existence of some overlays.\n";
    }
    
    // MARK: - MRMapViewDelegate
    
    func mapPickerDidPick(_ map : MRMap) {
        self.mapView.mapKey = map.key
    }
    
    func viewControllerToAddSheetTo(for mapView : MRMapView)->UIViewController {
        return self
    }
    
    func mapView(_ mapView : MRMapView, rendererFor overlay : MRPathOverlay)->MRPathRenderer {
        let renderer = MRPathRenderer(overlay: overlay)
        renderer.lineWidth = 7
        renderer.lineCap = .butt
        renderer.lineJoin = .bevel
        
        // if the renderer just has the default colors, intialize them specific to this Sample
        if(overlay == self.circleOverlay) {
            renderer.lineWidth = 5
            if(renderer.strokeColor != blueOpaqueStroke && renderer.strokeColor != redOpaqueStroke) {
                renderer.strokeColor = blueOpaqueStroke
                renderer.fillColor = blueTransparentFill
            }
        } else if (overlay == self.pathOverlay) {
            renderer.lineWidth = 4
            if(renderer.strokeColor != blueOpaqueStroke && renderer.strokeColor != redOpaqueStroke) {
                renderer.strokeColor = blueOpaqueStroke
                renderer.fillColor = clearFill
            }
        } else {
            renderer.lineWidth = 3
            if(renderer.strokeColor != blueTransparentStroke && renderer.strokeColor != redTransparentStroke) {
                renderer.strokeColor = blueTransparentStroke
                renderer.fillColor = blueTransparentFill
            }
        }
        return renderer
    }
    
    func mapView(_ mapView : MRMapView, didAddOverlayRenderers renderers :[MRPathRenderer]) {
        // noop
    }
    
    func mapView(_ mapView : MRMapView, viewFor annotation : MRAnnotation?)->MRAnnotationView? {
        if (annotation != nil && annotation is MRPlacemark) {
            let placemark = annotation as! MRPlacemark
            if (!placemark.area.isEmpty) {
    
                // A new MRPathOverlay should only be created if it does not already exist
                if placemarkOverlays[placemark.key] == nil {
                    let overlay = MRPathOverlay(placemark : placemark);
                    overlay.userInteractionEnabled = true;
                    self.mapView.add(overlay)
                    self.placemarkOverlays[placemark.key] = overlay;
                    self.mapView.renderer(for: overlay)
                }
            }
        }
        return nil;
    }
    
    func mapView(_ mapView : MRMapView, willTap overlay : MRPathOverlay)->MRPathOverlay {
        // allow the overlay to be tapped
        return overlay
    }
    
    func mapView(_ mapView : MRMapView, didTap overlay : MRPathOverlay) {
        let renderer : MRPathRenderer? = self.mapView.renderer(for : overlay)
        toggleStrokeColor(renderer!);
        toggleFillColor(renderer!);
    }
    
    
    // MARK: - Internal
    
    func toggleStrokeColor(_ renderer : MRPathRenderer) {
        if (renderer.strokeColor == blueOpaqueStroke ) {
            renderer.strokeColor = redOpaqueStroke;
        } else if(renderer.strokeColor == redOpaqueStroke) {
            renderer.strokeColor = blueOpaqueStroke;
        } else if(renderer.strokeColor == blueTransparentStroke) {
            renderer.strokeColor = redTransparentStroke;
        } else {
            renderer.strokeColor = blueTransparentStroke;
        }
    }
    
    func toggleFillColor(_ renderer : MRPathRenderer) {
        if (renderer.fillColor == clearFill ) {
            renderer.fillColor = clearFill;
        } else if(renderer.fillColor == redTransparentFill) {
            renderer.fillColor = blueTransparentFill;
        } else {
            renderer.fillColor = redTransparentFill;
        }
    }
    
    @objc func toggleColor() {
        // let overlays = self.mapView.overlays
        var overlays = [MRPathOverlay] ()
        overlays = self.mapView.overlays()
        for overlay in overlays {
            let renderer : MRPathRenderer? = self.mapView.renderer(for : overlay)
            toggleStrokeColor(renderer!);
            toggleFillColor(renderer!);
        }
    }
    
    @objc func toggleOverlay() {
        if let overlay = circleOverlay {
            self.mapView.remove(overlay)
            self.circleOverlay = nil
        } else {
            self.circleOverlay = MRPathOverlay(circleOfCenter: CGPoint(x: 200, y: 200), radius: 50)
            self.circleOverlay?.userInteractionEnabled = true
            self.mapView.add(self.circleOverlay!)
        }
        
        if let overlay = pathOverlay {
            self.mapView.remove(overlay)
            self.pathOverlay = nil
        } else {
            let points : Array <NSValue> = [ NSValue(cgPoint : CGPoint(x : 212, y : 298)),
                                             NSValue(cgPoint : CGPoint(x : 370, y : 550)),
                                             NSValue(cgPoint : CGPoint(x : 910, y : 240)),
                                             NSValue(cgPoint : CGPoint(x : 1058, y : 270)) ]
            self.pathOverlay = MRPathOverlay(polylineOfPoints : points)
            self.pathOverlay?.userInteractionEnabled = true
            self.mapView.add(self.pathOverlay!)
        }
    }
}
