//
//  MSPlacemarkListViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSPlacemarkListTableViewController: UITableViewController, MSExampleViewController, UIPickerViewDelegate, UIPickerViewDataSource {
    private lazy var picker: UIPickerView = {
        let picker = UIPickerView()
        picker.dataSource = self
        picker.delegate = self
        return picker
    }()

    private var request: MRPlacemarkRequest?
    private let placemarkTypes = ["all", "generic", "restroom", "stairs", "exit"]
    private var typeFilter = ""
    private var results = [MRPlacemarkResult]()

    override func viewDidLoad() {
        super.viewDidLoad()
        typeFilter = placemarkTypes[0]

        let button0 = UIBarButtonItem(title: "Filter", style: .plain, target: self, action: #selector(self.filterAction))
        self.navigationItem.rightBarButtonItems = [button0]

        self.loadPlacemarks()
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Placemark List"
    }

    static func exampleDetail() -> String {
        return "Example of how to make requests for placemarks from Meridian servers."
    }

    // MARK: - UITableViewController

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.results.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let reuseIdentifier = "MRPlacemarkListCell"

        var cell: UITableViewCell
        if let dequeueCell = tableView.dequeueReusableCell(withIdentifier: reuseIdentifier) {
            cell = dequeueCell
        } else {
            cell = UITableViewCell(style: .value1, reuseIdentifier: reuseIdentifier)
        }

        if indexPath.row < self.results.count {
            let result = self.results[indexPath.row]
            if let placemarkName = result.placemark.name, placemarkName.count > 0 {
               cell.textLabel?.text = placemarkName
            } else if let typeName = result.placemark.typeName, typeName.count > 0 {
                cell.textLabel?.text = typeName
            } else {
                cell.textLabel?.text = "Placemark"
            }
            cell.accessoryType = .disclosureIndicator
        }
        return cell
    }

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if indexPath.row < self.results.count {
            let result = self.results[indexPath.row]
            let controller = MRMapViewController()
            controller.pendingDestination = result.placemark
            self.navigationController?.pushViewController(controller, animated: true)
        }
    }

    // MARK: - UIPickerViewDelegate / UIPickerViewDatasource

    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }

    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return self.placemarkTypes.count
    }

    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        if row < self.placemarkTypes.count {
            return self.placemarkTypes[row]
        }
        return "array error..."
    }

    // MARK: - Action

    @objc private func filterAction() {
       let controller = UIViewController()
        controller.title = "Placemark Type"
        controller.view.backgroundColor = UIColor.white
        controller.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .done, target: self, action: #selector(self.dismissFilters))
        controller.view.addSubview(self.picker)

        let navController = UINavigationController.init(rootViewController: controller)
        self.present(navController, animated:true, completion:nil)
    }

    @objc private func dismissFilters() {
        self.dismiss(animated: true, completion: { [weak self] in
            guard let strongSelf = self else {
                return
            }
            let selectedIndex = strongSelf.picker.selectedRow(inComponent: 0)
            if selectedIndex < strongSelf.placemarkTypes.count {
                if strongSelf.typeFilter != strongSelf.placemarkTypes[selectedIndex] {
                    strongSelf.typeFilter = strongSelf.placemarkTypes[selectedIndex]
                    strongSelf.loadPlacemarks()
                }
            }
        })
    }

    // MARK: - Utilities

    private func loadPlacemarks() {
        self.refreshControl?.beginRefreshing()
        self.navigationItem.rightBarButtonItem?.isEnabled = false

        self.results.removeAll()
        self.request = MRPlacemarkRequest()
        self.request?.app = MREditorKey(identifier: MSExampleHost.appID())

        if let typeIndex = self.placemarkTypes.index(of: self.typeFilter), typeIndex > 0 {
            self.request?.filter = MRQueryFilter(field: "type", value: self.typeFilter)
        }

        self.request?.start(completionHandler: { [weak self] response, error in
            self?.placemarksLoadedWithResposne(response: response, error: error)
        })
    }

    private func placemarksLoadedWithResposne(response: MRPlacemarkResponse?, error: Error?) {
        if let requestError = error {
            print("Error loading placemarks: " + requestError.localizedDescription)
            return
        }

        if let responseResults = response?.results {
            self.results += responseResults
            self.tableView.reloadData()
        }

        if let nextPage = response?.nextPage {
            nextPage.start(completionHandler: { [weak self] nextResponse, nextError in
                self?.placemarksLoadedWithResposne(response: nextResponse, error: nextError)
            })
        } else {
            self.refreshControl?.endRefreshing()
            self.navigationItem.rightBarButtonItem?.isEnabled = true
        }
    }
}
